#include "cocoon.h"

void computeImage(C2PoincareMap3D& PM, GridSet3D& GS, FuncPointer fp, Frame& f, int iter, int color)
{
   GridSet3D::iterator b=GS.begin(), e=GS.end();
   while(b!=e)
   {
      Rect2Set3D s(*b,GS.C,GS.r);
      IVector3D result, x=IVector3D(s);
      drawVector(f,x,color);
      for(int k=1;k<=iter;++k)
      {
         if(k==iter)
            PM.setFactor(0.1);
         else
            PM.setFactor(1);
         result = PM(s);
      }
      drawVector(f,result,color);
      if(!fp(result,x))
         throw std::runtime_error("inclusion not satisfied");
      ++b;
   }
}

// ------------------------------------------------------------ //

void computeTimeMap(C2Taylor3D& DS, GridSet3D& GS, FuncPointer fp, Frame* f, int iter, int color)
{
   GridSet3D::iterator b=GS.begin(), e=GS.end();
   while(b!=e)
   {
      Rect2Set3D s(*b,GS.C,GS.r);
      for(int k=0;k<iter;++k)
         s.move(DS);
      IVector3D result=IVector3D(s);
      N.show(f[0],0,1,result,color);
      N.show(f[1],0,2,result,color);
      N.show(f[2],1,2,result,color);
      if(!fp(result,result))
         throw std::runtime_error("inclusion not satisfied");
      ++b;
   }
}

// ------------------------------------------------------------ //

IMatrix3D computeTimeMap(C2Taylor3D& DS, GridSet3D& GS, const IMatrix3D& M, const IMatrix3D& N, int iter)
{
   GridSet3D::iterator b=GS.begin(), e=GS.end();
   IMatrix3D DP;
   bool first = true;
   while(b!=e)
   {
      C1Rect23D s(*b,GS.C,GS.r,ENorm3D());
      for(int k=0;k<iter;++k)
         s.move(DS);
      if(first)
      {
         DP = N*IMatrix3D(s)*M;
         first = false;
      }else
         DP = IHull(DP,N*IMatrix3D(s)*M);
      ++b;
   }
   return DP;
}

// ------------------------------------------------------------ //

Matrix<interval,2,2> computeCompositionOfPoincareMap(
      const Matrix<interval,2,2>& DP1,
      const Matrix<interval,2,2>& DP2,
      Matrix<interval,2,2> D2P1[],
      Matrix<interval,2,2> D2P2[]
   )
{
   Matrix<interval,2,2> DQ, D2Q;
   DQ.clear();
   int i,j,k,c,ss;
   for(i=0;i<2;i++)
      for(j=0;j<2;j++)
         for(k=0;k<2;k++)
            DQ[i][j]  += DP2[i][k]*DP1[k][j];

   D2Q.clear();
   for(j=0;j<2;j++)
      for(c=0;c<2;c++)
         for(k=0;k<2;k++)
         {
            D2Q[j][c] += DP2[0][k] * D2P1[k][j][c];
            for(ss=0;ss<2;ss++)
            {
               D2Q[j][c] += D2P2[0][k][ss] * DP1[ss][c]*DP1[k][j];
            }
         }

   return D2Q;
}

// ------------------------------------------------------------ //

void computeCompositionOfPoincareMap(
      const Matrix<interval,2,2>& DP1,
      const Matrix<interval,2,2>& DP2,
      Matrix<interval,2,2> D2P1[],
      Matrix<interval,2,2> D2P2[],
      Matrix<interval,2,2>& DQ,
      Matrix<interval,2,2> D2Q[]
   )
{
   DQ.clear();
   int i,j,k,c,ss;
   for(i=0;i<2;i++)
      for(j=0;j<2;j++)
         for(k=0;k<2;k++)
            DQ[i][j]  += DP2[i][k]*DP1[k][j];

   D2Q[0].clear();
   D2Q[1].clear();
   for(i=0;i<2;++i)
      for(j=0;j<2;j++)
         for(c=0;c<2;c++)
            for(k=0;k<2;k++)
            {
               D2Q[i][j][c] += DP2[i][k] * D2P1[k][j][c];
               for(ss=0;ss<2;ss++)
               {
                  D2Q[i][j][c] += D2P2[i][k][ss] * DP1[ss][c]*DP1[k][j];
               }
            }
}

// ---------------------------------------------------------------- //

void IHullMatrix( const Matrix<interval,2,2>& DP, Matrix<interval,2,2> D2P[], 
                  Matrix<interval,2,2>& maxDP, Matrix<interval,2,2> maxD2P[]
                )
{
   for(int i=0;i<2;i++)
      for(int j=0;j<2;j++)
      {
         maxDP[i][j] = intervalHull(maxDP[i][j],DP[i][j]);
         for(int k=0;k<2;k++)
            maxD2P[k][i][j] = intervalHull(maxD2P[k][i][j],D2P[k][i][j]);
      }
}

// ---------------------------------------------------------------- //

void IHullMatrix(const IMatrix3D& DP, IMatrix3D D2P[],IMatrix3D& maxDP, IMatrix3D maxD2P[])
{
   for(int i=0;i<3;i++)
      for(int j=0;j<3;j++)
      {
         maxDP[i][j] = intervalHull(maxDP[i][j],DP[i][j]);
         for(int k=0;k<3;k++)
            maxD2P[k][i][j] = intervalHull(maxD2P[k][i][j],D2P[k][i][j]);
      }
}

// ---------------------------------------------------------------- //

void saveHalfPoincare(C2PoincareMap3D& PM, IVector3D X, C2Data& c)
{
   IMatrix3D DP, D2P[3];
   computePoincare(PM,X,DP,D2P,2);
   for(int i=0;i<2;i++)
   {
      c.iv[i]=X[i];
      for(int j=0;j<2;j++)
      {
         c.DP[i][j] = DP[i][j];
         for(int k=0;k<2;k++)
            c.D2P[k][i][j] = D2P[k][i][j];
      }
   }
}

// ---------------------------------------------------------------- //

void saveHalfPoincare(C2PoincareMap3D& PM, IVector3D X, IMatrix3D& C, IVector3D& r, C2Data& c)
{
   IMatrix3D DP, D2P[3];
   computePoincare(PM,X,C,r,DP,D2P,2);
   for(int i=0;i<2;i++)
   {
      c.iv[i]=X[i];
      for(int j=0;j<2;j++)
      {
         c.DP[i][j] = DP[i][j];
         for(int k=0;k<2;k++)
            c.D2P[k][i][j] = D2P[k][i][j];
      }
   }
}

// ---------------------------------------------------------------- //

void computeC2ImageOfBox(C2PoincareMap3D& PM, LinkC2Data& it, C2DataSet& halfPoincareData, double meshSize)
{
   IVector3D xGrid=IVector3D(interval(meshSize),interval(0.),interval(0.));
   IVector3D yGrid=IVector3D(interval(0.),interval(meshSize),interval(0.));
   IVector3D box=IVector3D(interval(0.,meshSize),interval(0.,meshSize),interval(0.));

   long leftIndexX  = static_cast<long>(it.iv[0].leftBound()/meshSize);
   long rightIndexX = static_cast<long>(it.iv[0].rightBound()/meshSize);
   long leftIndexY  = static_cast<long>((it.iv[1]-IyBif).leftBound()/meshSize);
   long rightIndexY = static_cast<long>((it.iv[1]-IyBif).rightBound()/meshSize);

   for(long xi=leftIndexX;xi<=rightIndexX;xi++)
      for(long yi=leftIndexY;yi<=rightIndexY;yi++)
      {
         if(halfPoincareData.find(intPair(xi,yi))==halfPoincareData.end())
         {
            IVector3D X = IVector3D(interval(0.),IyBif,interval(0.)) + xi*xGrid + yi*yGrid + box;
            C2Data c;
            saveHalfPoincare(PM,X,c);
            halfPoincareData[intPair(xi,yi)]=c;
         }
         it.link.insert(intPair(xi,yi));
      }
}
