#include "cocoon.h"

// --------------------------------------------------------------------- //

void init()
{
   double s = 1e-3;
   Vector<interval,2> center, stable,unstable;
   center[0]=0; center[1]=IyBif.mid();
   stable[0]=-.09; stable[1]=1;
   unstable[0]=-stable[0]; unstable[1]=stable[1];
   stable*=s;
   unstable*=s;
   H0 = HSet(center,stable,unstable);

   Vector<interval,2> A;
   A[0] = H0.IA[0] - 2e-4;
   A[1] = H0.IA[1] - 3e-3;
   Vector<interval,2> H1u = (H0.IA-A)/2;
   interval c = interval(3)/interval(5);
   Vector<interval,2> H1s = c*stable;
   Vector<interval,2> H1c = A + H1s + H1u;
   H1 = HSet(H1c,H1s,H1u);

   A[0] = H1.IA[0] - 9e-4;
   A[1] = H1.IA[1] - 8e-3;
   Vector<interval,2> H2u = (H1.IA-A)/2;
   Vector<interval,2> H2s = H1s;
   Vector<interval,2> H2c = A + H2s + H2u;
   H2 = HSet(H2c,H2s,H2u);

   Vector<interval,2> H3c,H3s,H3u;
   H3c[0]=-0.0038;
   H3c[1]=1.3337;
   H3s[0]=-1e-4; H3s[1]=2e-4;
   H3u[0]=1e-4; H3u[1]=4e-4;
   H3 = HSet(H3c,H3s,H3u);
   
   Vector<interval,2> H4c,H4s,H4u;
   H4s[0]=-1e-4; H4s[1]=2e-4;
   H4u[0]=3e-4; H4u[1]=5.5e-4;
   H4c[0] = -0.0335;
   H4c[1] = 1.2607;
   H4 = HSet(H4c,H4s,H4u);
   
   Vector<interval,2> H5c,H5s,H5u;
   H5s[0]=-1e-4; H5s[1]=2e-4;
   H5u[0]=3e-4; H5u[1]=5.5e-4;
   H5c[0] = -0.03798;
   H5c[1] = -3.10109;
   H5 = HSet(H5c,H5s,H5u);

   findEigenvectors(IcBif.mid().rightBound()); // here we take an approximate value of a center of C
   IVector3D Nc(-sqrt(2.)*IcBif.mid().rightBound(),0.,0.);
   N = HSet3D(Nc,.1*Iu,.2*Is1,.2*Is2);
}

// --------------------------------------------------------------------- //

void proveBifurcationPoint(C2PoincareMap4D& PM)
{
   rootFrame << "Proof of an existence of bifurcation point:\nCompute Newton operator - Lemma 9";
   PM.setOrder(10);
   PM.setStep(0.1);

   IVector4D x;
   x[1] = IyBif;
   x[3] = IcBif;
   IVector4D v = midVector(x);
   Vector<interval,2> V; V[0] = v[1]; V[1] = v[3]; // V i a center of the set
   Vector<interval,2> X; X[0] = x[1]; X[1] = x[3]; // X is the whole set
   IMatrix4D DP;
   IMatrix4D D2P[4];

   // we need to prove the existence of zero of the function
   // F(y,c) = (P_x(0,y,c), dP_x/dy(0,y,c))
   // we compute interval Newton operator for F
   // first we compute F(v) - at the center
   computePoincare(PM,v,DP,D2P,2);
   Vector<interval,2> Fv;
   Fv[0] = v[0];
   Fv[1] = DP[0][1];

   // we compute derivative of F on some larger set containing v, i.e. x
   computePoincare(PM,x,DP,D2P,2);

   // the following matrix contains a derivative of F on x
   Matrix<interval,2,2> DF;
   DF[0][0] = DP[0][3];
   DF[0][1] = DP[0][1];
   DF[1][0] = D2P[0][1][3];
   DF[1][1] = D2P[0][1][1];
   // we compute inverse matrix of DF
   interval det = DF[0][0]*DF[1][1] - DF[0][1]*DF[1][0];
   Matrix<interval,2,2> invDF;
   invDF[0][0] = DF[1][1];
   invDF[0][1] = -DF[0][1];
   invDF[1][0] = -DF[1][0];
   invDF[1][1] = DF[0][0];
   invDF /= det;

   // next we compute interval Newton operator
   // N = v - invDF(x) * F(v)
   Vector<interval,2> N = V - invDF * Fv;
   printVector(N);
   printVector(X);
   printVector(X-N);
   printVector(Fv);
   rootFrame <<  diam(N[0]) << "   " << diam(N[1]) << "\n";
   if(subset(N,X) && D2P[0][1][1]<0 && DP[0][3]<0)
   {
      std::ofstream plik;
      plik.open("BifurcationPoint.txt");
      plik.precision(25);
      plik << "Initial set:\n";
      plik << "yBif" << IyBif << "\n";
      plik << "cBif" << IcBif << "\n\n";
      plik << "Derivative of F at (c,y):\n";
      plik << "DF" << DF << "\n";
      plik << "Newton operator: N=" << N << "\n";
      plik.close();
      rootFrame << "ok\n";
   }
   else{
      throw std::runtime_error("Interval Newton operator - inlcusion not satisfied");
   }
}

// --------------------------------------------------------------------- //

void showC2Data(C2DataVector& c2DataSet, C2DataSet& halfPoincareData, std::set<intPair>& theSet, bool saveData=false)
{
   if(c2DataSet.size()==0) return;
   rootFrame << "total number of saved boxes in such a part: " << c2DataSet.size() 
             << "\ntotal number of saved boxes for half Poincare map in such a part: "  << theSet.size() << "            \n";

   C2DataVector::iterator b = c2DataSet.begin(), e=c2DataSet.end();
   Matrix<interval,2,2> DP1,DP2,D2P1[2],D2P2[2];
   DP1 = b->DP;
   D2P1[0] = b->D2P[0];
   D2P1[1] = b->D2P[1];
   ++b;
   while(b!=e)
   {
      IHullMatrix(b->DP,b->D2P,DP1,D2P1);
      ++b;
   }
   
   std::set<intPair>::iterator b1 = theSet.begin(), e1 = theSet.end();
   DP2 = halfPoincareData[*b1].DP;
   D2P2[0] = halfPoincareData[*b1].D2P[0];
   D2P2[1] = halfPoincareData[*b1].D2P[1];
   ++b1;
   while(b1!=e1)
   {
      IHullMatrix(halfPoincareData[*b1].DP,halfPoincareData[*b1].D2P,DP2,D2P2);
      ++b1;
   }
   Matrix<interval,2,2> D2X = computeCompositionOfPoincareMap(DP1,DP2,D2P1,D2P2);
   interval det = D2X[0][0]*D2X[1][1]-D2X[0][1]*D2X[1][0];
   rootFrame << "det(D2X)=" << det << "                   \n\n";

   if(!(det>0))
   {
      throw std::runtime_error("The existence of a Lyapunov function not verified!");
   }
   if(saveData)
   {
      std::ofstream plik;
      plik.open("C2BoundOnTrapezoid.txt",std::ios::app);
      plik.precision(20);
      plik << "D2X=" << D2X << "\n";
      plik << "det(D2X)=" << det << "\n\n";
      plik << "total number of saved boxes: " <<  c2DataSet.size() << ", " << halfPoincareData.size() << ", " << theSet.size() << "            \n";
      plik.close();
   }
}

// --------------------------------------------------------------------- //

void proveLyapunovFunction(C2PoincareMap3D& PM)
{
   rootFrame.Clear();
   PM.setOrder(8);
   PM.setStep(0.13);
   double meshSize = .1e-4;
   GridSet3D GS;
   H0.gridSet(GS,92,92);
   removeUnnecesaryBlocks(GS);

   C2DataVector leftBottom, leftTop;
   C2DataSet halfPoincareData;
   std::set<intPair> lB, lT;

   rootFrame << BgColor(BLACK) << FgColor(GREEN) 
             << "Proof of the existence of a Lyapunov function on the trapezoid - part of Lemma 18"
             << BgColor(WHITE) << FgColor(BLACK);
   for(unsigned int i=0;i<GS.center.size();i++)
   {
      rootFrame << At(3,0) << "Total number of boxes in a grid of the trapezoid: " << GS.center.size() <<  "       ";
      rootFrame << At(4,0) << "current box: " << i << "         \n\n";
      LinkC2Data c;
      IVector3D X = GS.center[i]+GS.C*GS.r;
      c.x[0] = X[0];
      c.x[1] = X[1];
      saveHalfPoincare(PM,GS.center[i],GS.C,GS.r,c);
      computeC2ImageOfBox(PM,c,halfPoincareData,meshSize);
      if(!(X[0]>0) && !(X[1]>IyBif))
      {
         lB.insert(c.link.begin(),c.link.end());
         leftBottom.push_back(c);
      }
      if(!(X[0]>0) && !(X[1]<IyBif))
      {
         lT.insert(c.link.begin(),c.link.end());
         leftTop.push_back(c);
      }

      rootFrame << BgColor(BLACK) << FgColor(GREEN) << "bottom part of the trapezoid:\n" << BgColor(WHITE) << FgColor(BLACK);
      showC2Data(leftBottom,halfPoincareData,lB);
      rootFrame << BgColor(BLACK) << FgColor(GREEN) << "top part of the trapezoid:\n" << BgColor(WHITE) << FgColor(BLACK);
      showC2Data(leftTop,halfPoincareData,lT);
   }

   std::ofstream plik;
   plik.open("C2BoundOnTrapezoid.txt");
   plik << "Bound of the hessian D^2 P^4_1 on bottom part of the trapezoid:\n\n";
   plik.close();
   showC2Data(leftBottom,halfPoincareData,lB,true);

   plik.open("C2BoundOnTrapezoid.txt",std::ios::app);
   plik << "Bound of the hessian D^2 P^4_1 on top part of the trapezoid:\n\n";
   plik.close();
   showC2Data(leftTop,halfPoincareData,lT,true);
}


// --------------------------------------------------------------------- //

void computeBoundaryOfH0(C2PoincareMap3D& PM)
{
// this function verifies that the unstable set of bifurcation point must leave a set 
// through a given small subset of the boundary

   rootFrame.Clear();
   PM.setOrder(10);
   PM.setStep(-0.1);

   double s=4e-3;
   coordSystem(rootFrame,s/3,s,0.,IyBif.rightBound());
   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "Computing preimage of the boundary of the trapezoid T - Eq.(54)" << BgColor(WHITE) << FgColor(BLACK);
   drawTrapezoid(rootFrame,RED);

   GridSet3D GS;
   H0.gridTopEdge(GS,1000);
   removeUnnecesaryBlocks(GS);
   computeImage(PM,GS,outsideH1H2Trapezoid,rootFrame,8,MAGENTA);

   PM.setStep(-0.25);
   PM.setOrder(12);
   std::vector<Rect2Set3D> GE;
   Vector<interval,2> L = H0.Icenter - H0.Iunstable + 2*H0.Istable/10;
   Vector<interval,2> R = H0.Icenter + H0.Istable   - 2*H0.Iunstable/10;
   Vector<interval,2> H = intervalHull(L,R);
   gridSet(GE,IVector3D(H[0],H[1],0),1,300);
   std::vector<Rect2Set3D>::iterator b1=GE.begin(), e1=GE.end();
   while(b1!=e1)
   {
      IVector3D v(*b1);
      drawVector(rootFrame,v,MAGENTA);
      IVector3D image;
      for(int i=0;i<2;i++) // second iteration of the Poincare map
      {
         PM.setFactor(1);
         PM(*b1);
         PM(*b1);
         PM(*b1);
         if(i==1)
            PM.setFactor(0.1);
         image = PM(*b1);
      }
      drawVector(rootFrame,image,BLUE);
      if(!outsideTrapezoid(image))
      {
         throw std::runtime_error("inclusion not satisfied");
      }
      b1++;
   }   
}

// ------------------------------------------------------------------------- //

void computeImageOfH1(C2PoincareMap3D& PM)
{
   rootFrame.Clear();
   PM.setOrder(8);
   PM.setStep(-0.1);

   double s=1e-2;
   coordSystem(rootFrame,s/4,s,0.,IyBif.rightBound());
   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "Computing preimage of the boundary of the set H1 - Eq.(54)" << BgColor(WHITE) << FgColor(BLACK);
   drawTrapezoid(rootFrame,RED);
   rootFrame << H1;

   GridSet3D GS;
   H1.gridBottomEdge(GS,300);   
   computeImage(PM,GS,outsideH1H2Trapezoid,rootFrame,4,BLUE);

   H1.gridTopEdge(GS,300);   
   computeImage(PM,GS,outsideH1H2Trapezoid,rootFrame,8,BLUE);

   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "\n\nThe existence of a Lyapunov function on H1 - part of Lemma 18" << BgColor(WHITE) << FgColor(BLACK);
   PM.setStep(0.2);
   PM.setOrder(10);
   int grid=20;
   int g1[]={5 , 5 , 7 , 10, 10, 10, 10, 10, 15, 15, 20, 20, 20, 20, 30, 30, 40,  40,  40,  40};
   int g2[]={20, 20, 20, 20, 25, 25, 30, 30, 35, 35, 50, 50, 80, 80, 90, 90, 100, 150, 300, 800};
   Vector<interval,2> u = H1.Iunstable/interval(grid);
   maxDelta = -1000;

   for(int i=0;i<grid;i++)
   {
      Vector<interval,2> c = H1.Icenter - H1.Iunstable + interval(2*i+1)*u;
      HSet T(c,H1.Istable,u);
      T.gridSet(GS,g1[i],g2[i]);
      computeImage(PM,GS,monotonicity,rootFrame,4,BLUE);
   }
   std::ofstream fileDelta;
   fileDelta.open("LyapunovFunctionOnH1.txt");
   fileDelta.precision(25);
   fileDelta << "P_1(v_1,v_2)-v_1 < " << maxDelta << " on the set H1";
   fileDelta.close();
}

// ------------------------------------------------------------------------- //

void computeImageOfH2(C2PoincareMap3D& PM)
{
   rootFrame.Clear();
   PM.setOrder(8);
   PM.setStep(-0.1);

   double s=2e-2;
   coordSystem(rootFrame,s/4,s,0.,IyBif.rightBound());
   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "Computing preimage of the boundary of the set H2 - Eq.(54)" << BgColor(WHITE) << FgColor(BLACK);
   drawTrapezoid(rootFrame,RED);
   rootFrame << H1 << H2;
   GridSet3D GS;

   H2.gridBottomEdge(GS,600);   
   computeImage(PM,GS,outsideH1H2Trapezoid,rootFrame,4,BLUE);

   H2.gridTopEdge(GS,200);   
   computeImage(PM,GS,outsideH1H2Trapezoid,rootFrame,4,BLUE);

   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "\n\nThe existence of a Lyapunov function on H2 - part of Lemma 18" << BgColor(WHITE) << FgColor(BLACK);
   PM.setStep(0.1);
   H2.gridSet(GS,250,20);
   maxDelta = -1000;
   computeImage(PM,GS,monotonicity,rootFrame,4,BLUE);
   std::ofstream fileDelta;
   fileDelta.open("LyapunovFunctionOnH2.txt");
   fileDelta.precision(25);
   fileDelta << "P_1(v_1,v_2)-v_1 < " << maxDelta << " on the set H2";
   fileDelta.close();
}

// ------------------------------------------------------------------------- //

void coverH2H3(C2PoincareMap3D& PM)
{
   rootFrame.Clear();
   PM.setOrder(8);
   PM.setStep(0.1);

   double s=2.5e-2;
   coordSystem(rootFrame,s/4,s,H3.center[0],H3.center[1]);
   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "\n\nThe existence of covering relation H2->H3 - Eq.(45)" << BgColor(WHITE) << FgColor(BLACK);
   drawTrapezoid(rootFrame,RED);
   rootFrame << H1 << H2 << H3;
   GridSet3D GS;

   H2.gridBottomEdge(GS,700);   
   computeImage(PM,GS,acrossH3,rootFrame,4,GREEN);
   H2.gridTopEdge(GS,350);   
   computeImage(PM,GS,acrossH3,rootFrame,4,GREEN);
   H2.gridLeftEdge(GS,5);   
   computeImage(PM,GS,leftH3,rootFrame,4,RED);
   H2.gridRightEdge(GS,5);   
   computeImage(PM,GS,rightH3,rootFrame,4,BLUE);
}

// ------------------------------------------------------------------------- //

void coverH3H4(C2PoincareMap3D& PM)
{
   rootFrame.Clear();
   PM.setOrder(8);
   PM.setStep(0.1);

   double s=1e-2;
   coordSystem(rootFrame,s/4,s,H4.center[0],H4.center[1]);
   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "\n\nThe existence of covering relation H3->H4 - Eq.(45)" << BgColor(WHITE) << FgColor(BLACK);
   rootFrame << H4;
   GridSet3D GS;

   H3.gridBottomEdge(GS,80);   
   computeImage(PM,GS,acrossH4,rootFrame,4,GREEN);
   H3.gridTopEdge(GS,80);   
   computeImage(PM,GS,acrossH4,rootFrame,4,GREEN);
   H3.gridLeftEdge(GS,3);   
   computeImage(PM,GS,leftH4,rootFrame,4,RED);
   H3.gridRightEdge(GS,3);   
   computeImage(PM,GS,rightH4,rootFrame,4,BLUE);
}

// ------------------------------------------------------------------------- //

void coverH4H5(C2PoincareMap3D& PM)
{
   rootFrame.Clear();
   PM.setOrder(8);
   PM.setStep(0.1);

   double s=2e-3;
   coordSystem(rootFrame,s/4,s,H5.center[0],H5.center[1]);
   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "\n\nThe existence of covering relation H4->H5 - Eq.(45)" << BgColor(WHITE) << FgColor(BLACK);
   rootFrame << H5;
   GridSet3D GS;

   H4.gridBottomEdge(GS,40);   
   computeImage(PM,GS,acrossH5,rootFrame,2,GREEN);
   H4.gridTopEdge(GS,40);   
   computeImage(PM,GS,acrossH5,rootFrame,2,GREEN);
   H4.gridLeftEdge(GS,7);   
   computeImage(PM,GS,leftH5,rootFrame,2,RED);
   H4.gridRightEdge(GS,7);   
   computeImage(PM,GS,rightH5,rootFrame,2,BLUE);
}

// ------------------------------------------------------------------------- //

void coverH5N(C2Taylor3D& DS)
{

   Frame f[3];
   coordSystem3D(f,3);
   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "\n\nThe existence of covering relation H5->M - Eq.(45)" << BgColor(WHITE) << FgColor(BLACK);

   DS.setOrder(8);
   DS.setStep(interval(1)/interval(10));
   int numberOfSteps = 70; // Phi_T, where T=70*0.1=7

   GridSet3D GS;
   H5.gridSet(GS,20,1);
   computeTimeMap(DS,GS,acrossN,f,numberOfSteps,GREEN);
   H5.gridLeftEdge(GS,1);   
   computeTimeMap(DS,GS,leftN,f,numberOfSteps,RED);
   H5.gridRightEdge(GS,1);   
   computeTimeMap(DS,GS,rightN,f,numberOfSteps,BLUE);
}

// ------------------------------------------------------------------------- //

void coverN(C2Taylor3D& DS)
{
   Frame f[3];
   coordSystem3D(f,3);
   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "\n\nThe existence of covering relation M->M - Eq.(45)" << BgColor(WHITE) << FgColor(BLACK);

   DS.setOrder(8);
   DS.setStep(interval(1)/interval(10));
   int numberOfSteps = 14; // Phi_T, where T=14*0.1=1.4

   GridSet3D GS;
   N.gridWall(GS,20,20,1);
   computeTimeMap(DS,GS,acrossN,f,numberOfSteps,GREEN);
   N.gridWall(GS,20,20,2);
   computeTimeMap(DS,GS,acrossN,f,numberOfSteps,GREEN);
   N.gridWall(GS,10,10,0,-1);
   computeTimeMap(DS,GS,leftN,f,numberOfSteps,BLUE);
   N.gridWall(GS,10,10,0,1);
   computeTimeMap(DS,GS,rightN,f,numberOfSteps,RED);
}

// ------------------------------------------------------------------------- //

void proveConeConditions(C2PoincareMap3D& PM, HSet& N, HSet& M, int grid1, int grid2,int iter)
{   
   Matrix<interval,2,2> CN, CM, DPc, Q;
   CN.column(0) = N.Iunstable;
   CN.column(1) = N.Istable;
   CM.column(0) = M.Iunstable;
   CM.column(1) = M.Istable;
   CM = inverseMatrix(CM);
   Q(1,1) = 1; Q(2,2)=-1; // quadratic form is x^2-y^2
   
   GridSet3D GS;
   N.gridSet(GS,grid1,grid2);
   GridSet3D::iterator b = GS.begin(), e=GS.end();

   bool first = true;
   while(b<e)
   {
      IMatrix3D DP;
      computePoincare(PM,*b,GS.C,GS.r,DP,iter);
      Matrix<interval,2,2> DP1;
      DP1(1,1) = DP(1,1);
      DP1(1,2) = DP(1,2);
      DP1(2,1) = DP(2,1);
      DP1(2,2) = DP(2,2);
      Matrix<interval,2,2> temp = CM * DP1 * CN;
      if(first)
      {
         DPc = Transpose(temp)*Q*temp-Q;
         first = false;
      } else
         DPc = IHull(DPc,Transpose(temp)*Q*temp-Q);
      ++b;
   }
   verifyQuadraticForm(DPc);
}

// ------------------------------------------------------------------------- //

void proveConeConditions(C2Taylor3D& DS, C2PoincareMap3D& PM)
{
   // we compute df_c^T*Q_N*df_c - Q_N on hyperbolic set N around equilibrium

   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "Cone condition for covering relation M->M\n" << BgColor(WHITE) << FgColor(BLACK);
   DS.setStep(interval(1)/interval(10));
   DS.setOrder(10);
   int numberOfSteps = 14; // Phi_T, where T=14*0.1 = 1.4

   GridSet3D GS;
   N.gridSet(GS,12,12,12);
   IMatrix3D M = N.IM;
   IMatrix3D invM = inverseMatrix(M);
   IMatrix3D DP = computeTimeMap(DS,GS,M,invM,numberOfSteps);

   interval alpha=1, beta=-1;
   IMatrix3D QN;
   QN(1,1) = alpha; QN(2,2)=beta; QN(3,3)=beta; // quadratic form is x^2-y^2-z^2
   verifyQuadraticForm(Transpose(DP)*QN*DP-QN);

// ------------

   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "Cone condition for covering relation H5->M\n" << BgColor(WHITE) << FgColor(BLACK);
   numberOfSteps = 70;

   GridSet3D GS2;
   H5.gridSet(GS2,1,1);
   Matrix<interval,2,2> QH;
   QH(1,1)=alpha; QH(2,2)=beta;
   IMatrix3D NH5 = GS.C;
   DP = computeTimeMap(DS,GS2,IMatrix3D::Identity(3),IMatrix3D::Identity(3),numberOfSteps);
   Matrix<interval,3,2> dF;
   dF.column(0) = DP.column(0);
   dF.column(1) = DP.column(1);

   // coordinate system on H5
   Matrix<interval,2,2> PH5;
   PH5.column(0) = H5.Iunstable;
   PH5.column(1) = H5.Istable;

   // we compute the derivative of dFc, i.e. we choose proper coordinate systems
   Matrix<interval,3,2> dFc = invM * dF * PH5;
   verifyQuadraticForm(Transpose(dFc)*QN*dFc - QH);

// ------------

   PM.setOrder(10);
   PM.setStep(0.1);

   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "Cone condition for covering relation H4->H5\n" << BgColor(WHITE) << FgColor(BLACK);
   proveConeConditions(PM,H4,H5,1/*grid1*/,1/*grid2*/,2/*iterations of the Poincare Map*/);

   rootFrame << BgColor(BLACK) << FgColor(GREEN) << "Cone condition for covering relation H3->H4\n" << BgColor(WHITE) << FgColor(BLACK);
   proveConeConditions(PM,H3,H4,6/*grid1*/,3/*grid2*/,4/*iterations of the Poincare Map*/);
}


// ------------------------------------------------------------------------- //

void computeDPAtBifPoint(C2PoincareMap3D& PM)
{
   IVector3D X(interval(0.),IyBif,interval(0.));
   IMatrix3D DP, D2P[3];
   PM.setOrder(10);
   PM.setStep(0.0625);
   computePoincare(PM,X,DP,D2P,4);
   std::ofstream dp;
   dp.open("DP_atBifPoint.txt");
   dp.precision(20);
   dp << "Rigorous bound for the derivative of Poincare map at the bifurcation point:\n\n";
   dp << DP[0][0] << "&" << DP[0][1] << "\\\\\n";
   dp << DP[1][0] << "&" << DP[1][1] << "\n";
   dp.close();
}

// ------------------------------------------------------------------------- //

int main(int, char *[])
{

   openGW(1000,700,WHITE,BLACK);
   
   try{
      std::string vField4D="var:x,y,z,c;fun:y,z,c^2-y-0.5*x*x,0;";
      std::string vField3D="var:x,y,z,c;fun:y,z,c^2-y-0.5*x*x;";
      std::string section3D="var:x,y,z;fun:z;";
      std::string section4D="var:x,y,z,c;fun:z;";
      C2Taylor3D DS3D(vField3D,8,0.01);
      C2Taylor4D DS4D(vField4D,8,0.01);
      C2PoincareMap3D PM3D(vField3D,section3D,10,0.1);
      C2PoincareMap4D PM4D(vField4D,section4D,10,0.1);

      DS3D.setParameter("c",IcBif);
      PM3D.setParameter("c",IcBif);
      long czas = time(0);
      init();

      proveBifurcationPoint(PM4D);
      proveConeConditions(DS3D, PM3D);
      computeDPAtBifPoint(PM3D);
      proveLyapunovFunction(PM3D);
      computeBoundaryOfH0(PM3D);
      computeImageOfH1(PM3D);
      computeImageOfH2(PM3D);
      coverH2H3(PM3D);
      coverH3H4(PM3D);
      coverH4H5(PM3D);
      coverH5N(DS3D);
      coverN(DS3D);
          
      rootFrame << At(35,0) << "time of computations (in seconds) = " << time(0)-czas << "\n";
   } catch(std::exception &e)
   {
      rootFrame << "\n\n" << BgColor(BLACK) << FgColor(ORANGE) << "Exception caught! See 'exception.txt' file for details.\n";
      std::ofstream plik;
      plik.open("exception.txt");
      plik << e.what();
      plik.close();
   }
   waitBt();
   closeGW();

   return 0;
}
