#include "cocoon.h"

bool outsideTrapezoid(const IVector3D& iv)
{
   const static Vector<interval,2> L = H0.Icenter - H0.Iunstable + 2*H0.Istable/10;
   const static Vector<interval,2> R = H0.Icenter + H0.Istable   - 2*H0.Iunstable/10;
   const static Vector<interval,2> H = intervalHull(L,R);
   Vector<interval,2> Z;
   Z[0]=iv[0]; Z[1]=iv[1];
   return (Z[0] < H[0]) || (Z[0]>0) || H0.outside(Z);
}

bool outsideTrapezoid(const Vector<interval,2>& Z)
{
   const static Vector<interval,2> L = H0.Icenter - H0.Iunstable + 2*H0.Istable/10;
   const static Vector<interval,2> R = H0.Icenter + H0.Istable   - 2*H0.Iunstable/10;
   const static Vector<interval,2> H = intervalHull(L,R);
   return (Z[0] < H[0]) || (Z[0]>0) || H0.outside(Z);
}

// ------------------------------------------------------------------------ // 

bool empty(const IVector3D&, const IVector3D&)
{
   return true;
}

bool outsideH1H2Trapezoid(const IVector3D& iv, const IVector3D&)
{
   Vector<interval,2> Z;
   Z[0]=iv[0]; Z[1]=iv[1];
   return H1.outside(Z) && H2.outside(Z) && outsideTrapezoid(Z);
}

bool monotonicity(const IVector3D& image, const IVector3D& X)
{
   static long current = 0;
   if(maxDelta==-1000) current=0;
   interval diff = image[0] - X[0];
   maxDelta = capd::max(maxDelta,diff.rightBound());
   current++;
   if(!(current % 20) )
   {
      rootFrame << At(8,0) << "Current box: " << current << "     ";
      rootFrame << At(9,0) << "P_1(v1,v2)-v1 < " << maxDelta << "               ";
   }
   return (diff < 0) && outsideTrapezoid(image);
}

// ------------------------------------------------------------------------ // 

bool leftH3(const IVector3D& iv, const IVector3D&)
{
   Vector<interval,2> Z;
   Z[0]=iv[0]; Z[1]=iv[1];
   return H3.onLeft(Z);
}

bool rightH3(const IVector3D& iv, const IVector3D&)
{
   Vector<interval,2> Z;
   Z[0]=iv[0]; Z[1]=iv[1];
   return H3.onRight(Z);
}

bool acrossH3(const IVector3D& iv, const IVector3D&)
{
   Vector<interval,2> Z;
   Z[0]=iv[0]; Z[1]=iv[1];
   return H3.across(Z);
}

// ------------------------------------------------------------------------ // 

bool leftH4(const IVector3D& iv, const IVector3D&)
{
   Vector<interval,2> Z;
   Z[0]=iv[0]; Z[1]=iv[1];
   return H4.onLeft(Z);
}

bool rightH4(const IVector3D& iv, const IVector3D&)
{
   Vector<interval,2> Z;
   Z[0]=iv[0]; Z[1]=iv[1];
   return H4.onRight(Z);
}

bool acrossH4(const IVector3D& iv, const IVector3D&)
{
   Vector<interval,2> Z;
   Z[0]=iv[0]; Z[1]=iv[1];
   return H4.across(Z);
}

// ------------------------------------------------------------------------ // 

bool leftH5(const IVector3D& iv, const IVector3D&)
{
   Vector<interval,2> Z;
   Z[0]=iv[0]; Z[1]=iv[1];
   return H5.onLeft(Z);
}

bool rightH5(const IVector3D& iv, const IVector3D&)
{
   Vector<interval,2> Z;
   Z[0]=iv[0]; Z[1]=iv[1];
   return H5.onRight(Z);
}

bool acrossH5(const IVector3D& iv, const IVector3D&)
{
   Vector<interval,2> Z;
   Z[0]=iv[0]; Z[1]=iv[1];
   return H5.across(Z);
}

// ------------------------------------------------------------------------ // 

bool leftN(const IVector3D& iv, const IVector3D&)
{
   return N.onLeft(iv);
}

bool rightN(const IVector3D& iv, const IVector3D&)
{
   return N.onRight(iv);
}

bool acrossN(const IVector3D& iv, const IVector3D&)
{
   return N.across(iv);
}
