/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file c0set.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _SET_H_
#define _SET_H_

#include <string>
#include <sstream>
#include "dynsys/dynsysintf.h"

namespace capd{
namespace dynset{

template<typename MatrixT>
class C0set{
public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;

   virtual ~C0set (void) {}
   virtual C0set *clone(void) = 0;
   virtual C0set *fresh(void) = 0;
   virtual C0set *cover(const VectorType &v)  = 0;
   virtual ScalarType size(void) = 0;
   virtual VectorType center(void) = 0;
   virtual void move(capd::dynsys::DynSys<MatrixType>& dynsys) = 0;
   virtual VectorType affineTransformation(const MatrixType&, const VectorType&) const = 0;

   virtual std::string show(void) const = 0;
   virtual operator VectorType() const = 0;
};

}} //namespace capd::dynset

#endif // #define _SET_H_

/// @}
