/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file c2set.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2003-2004 */

#ifndef _C2setClass
#define _C2setClass

#include <string>

#include "dynsys/c2dynsys.h"
#include "vectalg/normintf.h"

namespace capd{
namespace dynset{

template<typename MatrixT>
class C2set{
public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;

   virtual C2set *clone(void) const = 0;
   virtual C2set *fresh(void) const = 0;
   virtual C2set *cover(const VectorType &v) const = 0;
   virtual ScalarType size(void) const = 0;
   virtual VectorType center(void) const = 0;
   virtual void move(capd::dynsys::C2DynSys<MatrixType> &c2dynsys) = 0;
   virtual NormType *getNorm(void) = 0;

   virtual std::string show(void) const;
   virtual operator VectorType() const = 0;
   virtual operator MatrixType() const = 0;
   virtual const ScalarType& operator()(int i, int j, int c) const = 0;
   virtual VectorType operator()(int j, int c) const = 0;
   virtual MatrixType operator()(int i) = 0;
   virtual ~C2set(){}
};


template<typename MatrixType>
inline std::string C2set<MatrixType>::show(void) const
{
   return "This is an unkonown set";
}

}} // the end of the namespace capd::dynset

#endif // _C2setClass

/// @}
