/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file cnset.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2006 */

#ifndef _CnSetClass
#define _CnSetClass

#include "dynsys/cndynsys.h"
#include "vectalg/normintf.h"
#include "vectalg/multiindex.h"

namespace capd{
namespace dynset{

template<typename MatrixT>
class CnSet{
public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef typename MatrixType::RefColumnVectorType RefVectorType;
   typedef Norm<VectorType,MatrixType> NormType;
   typedef capd::vectalg::Multipointer Multipointer;
   typedef capd::vectalg::Multiindex Multiindex;
   typedef capd::map::CnCoeff<MatrixType> CnCoeffType;

   virtual CnSet* clone(void) const = 0;
   virtual ScalarType size(void) const = 0;
   virtual VectorType center(void) const = 0;
   virtual void move(capd::dynsys::CnDynSys<MatrixType>& cndynsys) = 0;
   virtual NormType* getNorm(void) = 0;
   virtual int getRank() const = 0;
   virtual int getDimension() const = 0;
   virtual const CnCoeffType& currentSet() const = 0;
   virtual CnCoeffType& currentSet() = 0;
   
   virtual operator VectorType() const = 0;
   virtual operator MatrixType() const = 0;
   // this operators returns d^{mp}f_i
   virtual const ScalarType& operator()(int i,const Multiindex& mp) const = 0;
   // this operator returns a vector of partial derivatives, i.e. result[i] = d^{mp}f_i
   virtual RefVectorType operator()(const Multipointer& mp) const = 0;
   // indexing for C^2
   virtual const ScalarType& operator()(int i, int j, int c) const = 0;

   // indexing for C^1   
   virtual const ScalarType& operator()(int j, int c) const = 0;
   // indexing for C^0
   virtual const ScalarType& operator()(int i) const = 0;
   virtual ~CnSet(){}
};

}} // the end of the namespace capd::dynset

#endif // _CnSetClass

/// @}
