/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file c1dynsys.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _C1DynSys_H_
#define _C1DynSys_H_

#include <string>
#include "dynsys/dynsysintf.h"

namespace capd{
namespace dynsys{

template<typename MatrixType>
class C1DynSys : public virtual capd::dynsys::DynSys<MatrixType>{
public:
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;

   virtual void JacRemainder(
            const VectorType &vecEnclosure,
            const MatrixType &jacEnclosure,
            VectorType &Remainder,
            MatrixType &jRemainder
      ) const = 0;
   virtual MatrixType jacEnclosure( const VectorType& enc, NormType& norm) const = 0;
   virtual VectorType enclosure(const VectorType& x, int* found) const = 0;

   using DynSys<MatrixType>::Phi;
   using DynSys<MatrixType>::JacPhi;
   using DynSys<MatrixType>::Lipschitz;
   using DynSys<MatrixType>::type;
};

}} // namespace capd::dynsys

#endif

/// @}
