/// @addtogroup map
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file funcintf.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _FUNCINTF_H
#define _FUNCINTF_H

#include <string>
#include "map/BasicFunctionIntf.h"

namespace capd{
namespace map{

/* ________________________ CLASS FUNCTION _______________________*/

template<typename VectorType>
class Function : protected BasicFunction<typename VectorType::ScalarType>
{
public:
   typedef typename VectorType::ScalarType ScalarType;

   Function(const char *, int order = 0); /* constructors */
   Function(const std::string &, int order = 0);
   Function(void);
   Function(const Function &);
   ~Function(void);

   const std::string& formula(void) const {return m_fun;}    /* returns the formula as a std::string */
   const std::string& partialDerivative(int i) const;      /* returns partial derivative as a std::string */
   std::string gradient(int) const;

   void setOrder (int order);

   Function& operator= (const char *);
   Function& operator= (const std::string &);
   Function& operator= (const Function &);

   std::string operator[] (int) const;         /* partial derivative with respect to i-th coordinate 0<=i<dim */

   ScalarType operator() (const ScalarType& v) const;         /* for 1-dim function */
   ScalarType operator() (const VectorType &) const ;
   ScalarType operator() (const VectorType &v, int i) const;
//   const ScalarType* const computeDerivatives(const VectorType &v, int a_upTo) const; 

   using BasicFunction<ScalarType>::setParameter;
   using BasicFunction<ScalarType>::variables;
   using BasicFunction<ScalarType>::dimension;

protected:
   using BasicFunction<ScalarType>::m_dim;
   using BasicFunction<ScalarType>::m_order;
   using BasicFunction<ScalarType>::m_val;
   using BasicFunction<ScalarType>::m_var;
   using BasicFunction<ScalarType>::m_size;
   using BasicFunction<ScalarType>::diffanal;
   using BasicFunction<ScalarType>::eqnanal;

   std::string* m_diff; /* the table of partial derivatives */
   std::string m_fun;   /* formula of the function */
   Node<ScalarType>* m_tree;

   void createFromText(const std::string &, int);
   void createDefault();
   void createFromObject(const Function &);
   void deleteTables();
   const std::string& currentFormula() {return m_fun;} 
};

}} // the end of the namespace capd::map

#endif //define _FUNCINTF_H

/// @}
