/// @addtogroup map
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file mapintf.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak,  2000-2005 */

#ifndef _MAPINTF_H
#define _MAPINTF_H

#include <string>
#include <stdexcept>
#include <sstream>
#include <vector>
#include "map/funcintf.h"

namespace capd{
namespace map{

template<typename MatrixT>
class Map
{
public:
   Map(void);
   Map(const std::string &, int order=0);
   Map(const char *, int order=0);
   Map(const Map &);

   Map &operator=(const char *);
   Map &operator=(const std::string &);
   Map &operator=(const Map &);
   virtual ~Map()
   {
      delete []fun;
      delete []der;
   }

   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename VectorType::ScalarType ScalarType;
   typedef Function<VectorType> FunctionType;

   VectorType operator()(const VectorType &, int = 0) const;
   VectorType operator()(const VectorType &, MatrixType &) const;
   MatrixType operator[](const VectorType &) const;
   const FunctionType &operator()(int) const;

   void variational(VectorType[], MatrixType[], int i) const;
   std::string variables() const {
      return fun[0].variables();
   }

   virtual void setParameter(const std::string &name, const ScalarType& value);
   virtual void setOrder(int the_new);
   int getDimension() const{
      return dim;
   }

protected:
   int dim, order;
   FunctionType *fun;
   FunctionType *der; // partial derivatives of a map, in fact dim*dim functions

private:
   void createFromText(const std::string &,int);
   void createFromObject(const Map &);
}; // the end of class Map


/*________________________________________________*/

template<typename MatrixType>
inline const typename Map<MatrixType>::FunctionType& Map<MatrixType>::operator()(int i) const
{
   return fun[i-1];
}

template<typename MatrixType>
inline  typename Map<MatrixType>::VectorType Map<MatrixType>::operator()(const VectorType &x, MatrixType &dF) const
{
   dF = operator[](x);
   return operator()(x);
}


}} // the end of the namespace capd::map

#endif //define _MAPINTF_H

/// @}
