/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file factrial.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <cstddef>
#include "dynsys/factrial.h"


static long *factorial_storage=NULL;
static long first_unknown_factorial=0;
static long *newton_storage=NULL;
static long first_unknown_newton_level=0;

inline long index(long n,long k)
{
	return n*(n+1)/2+k;
}

long factorial(long n)
{
	if(n<first_unknown_factorial){
		return factorial_storage[n];
	}else{
		long i;
		long *new_storage=new long[n+1];
		for(i=0;i<first_unknown_factorial;i++) new_storage[i]=factorial_storage[i];
		if(first_unknown_factorial == 0){
			new_storage[first_unknown_factorial++]=1;
		}
		long result=new_storage[first_unknown_factorial-1];
		for(i=first_unknown_factorial;i<=n;i++) new_storage[i]=result*=i;
		delete factorial_storage;
		factorial_storage=new_storage;
		first_unknown_factorial=n+1;
		return result;
	}
}

long newton(long n, long k)
{
	int first_undefined_index=index(first_unknown_newton_level,0);
	if(n>=first_unknown_newton_level){
		long *new_storage=new long[index(n+1,0)];
		for(long i=0;i<first_undefined_index;i++) new_storage[i]=newton_storage[i];
		if(first_undefined_index == 0){
			new_storage[first_undefined_index++]=1;
			first_unknown_newton_level++;
		}
		for(int m=first_unknown_newton_level;m<=n;m++){
			new_storage[first_undefined_index]=
			new_storage[first_undefined_index+m]=1;
			for(int p=1;p<m;p++) new_storage[first_undefined_index+p]=
					new_storage[index(m-1,p-1)]+new_storage[index(m-1,p)];
			first_undefined_index+=(m+1);
		}
		first_unknown_newton_level=n+1;
		delete newton_storage;
		newton_storage=new_storage;
	}
	return newton_storage[index(n,k)];
}

/// @}
