// @addtogroup rounding
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file doubleRounding.cpp
///
/// @author CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include "rounding/doubleRounding.h"

namespace capd{
namespace rounding{

void DoubleRounding::initFpUnit()
{
//std::cout << "\n fpInit";
#if(defined IBM) // this could also be implemented for LINUX and SPARC but so far was not needed
    // For some reason Win98 turns off some of the error handling of INTEL processors,
    // so we first turn on all numerical error handling

#if defined(BORLAND)          // Windows and Borland C++
    asm              fninit;
#else                        // Windows and GNU C++
    asm volatile (
        "fninit\n\t"
    );
#endif
    
// next we turn on/off "forbidden numeric operation" error handling for INTEL processors
#if(defined (INTEL_IM_CLEAR))
   clear_intel_im();
#else
   set_intel_im();
#endif
  
#endif //defined(IBM)

}

 
DoubleRounding::DoubleRounding()
{
   initFpUnit();
}
   
RoundingMode DoubleRounding::test()
{
  double a,b,nexta,preva,nextb,eps;
  RoundingMode result= RoundUnknown;

  nexta=a=1.5;
  do{ 
    preva=a;
    a=nexta;
    nexta=(a-1.0)/2+1.0;
  }while(a!=nexta);

  nextb=-1.5;
  do{
    b=nextb;
    nextb=(b+1.0)/2-1.0;
  }while(b!=nextb);

  eps=((2*(preva-1))/3+1);
  
  if(a == 1 && b == -1 && eps == 1)
  {
    result=RoundCut;
  }
  else if(a>1  && (preva-1)/3>0)
  {
    result=RoundUp;
  }
  else if(b<-1)
  {
    result=RoundDown;
  }
  else
  {
    result=RoundNearest;
  }
  
  return result;

}

}} // namespace capd::rounding

capd::rounding::DoubleRounding initRounding;
